@@ -10,10 +10,11 @@
 #include <ctype.h>
 #include <linux/fs.h>
 
-#include "asm/types.h"
-#include "list.h"
+#include "types.h"
+#include "common/list.h"
 #include "util.h"
 #include "mount.h"
+#include "filesystems.h"
 #include "mman.h"
 #include "cpu.h"
 #include "file-lock.h"
@@ -25,6 +26,7 @@
 #include "vma.h"
 #include "bfd.h"
 #include "proc_parse.h"
+#include "fdinfo.h"
 #include "parasite.h"
 #include "cr_options.h"
 #include "sysfs_parse.h"
@@ -33,6 +35,9 @@
 #include "namespaces.h"
 #include "files-reg.h"
 #include "cgroup.h"
+#include "cgroup-props.h"
+#include "timerfd.h"
+#include "path.h"
 
 #include "protobuf.h"
 #include "images/fdinfo.pb-c.h"
@@ -40,6 +45,19 @@
 
 #include <stdlib.h>
 
+#ifndef SIGEV_SIGNAL
+#define SIGEV_SIGNAL    0       /* notify via signal */
+#endif
+#ifndef SIGEV_NONE
+#define SIGEV_NONE      1       /* other notification: meaningless */
+#endif
+#ifndef SIGEV_THREAD
+#define SIGEV_THREAD    2       /* deliver via thread creation */
+#endif
+#ifndef SIGEV_THREAD_ID
+#define SIGEV_THREAD_ID 4       /* deliver to thread */
+#endif
+
 struct buffer {
 	char buf[PAGE_SIZE];
 	char end; /* '\0' */
@@ -82,8 +100,6 @@ static bool is_vma_range_fmt(char *line)
 static int parse_vmflags(char *buf, struct vma_area *vma_area)
 {
 	char *tok;
-	bool shared = false;
-	bool maywrite = false;
 
 	if (!buf[0])
 		return 0;
@@ -95,12 +111,6 @@ static int parse_vmflags(char *buf, struct vma_area *vma_area)
 #define _vmflag_match(_t, _s) (_t[0] == _s[0] && _t[1] == _s[1])
 
 	do {
-		/* open() block */
-		if (_vmflag_match(tok, "sh"))
-			shared = true;
-		else if (_vmflag_match(tok, "mw"))
-			maywrite = true;
-
 		/* mmap() block */
 		if (_vmflag_match(tok, "gd"))
 			vma_area->e->flags |= MAP_GROWSDOWN;
@@ -144,12 +154,6 @@ static int parse_vmflags(char *buf, struct vma_area *vma_area)
 
 #undef _vmflag_match
 
-	if (shared && maywrite)
-		vma_area->e->fdflags = O_RDWR;
-	else
-		vma_area->e->fdflags = O_RDONLY;
-	vma_area->e->has_fdflags = true;
-
 	if (vma_area->e->madv)
 		vma_area->e->has_madv = true;
 
@@ -175,12 +179,48 @@ static inline int vfi_equal(struct vma_file_info *a, struct vma_file_info *b)
 			(a->dev_min ^ b->dev_min)) == 0;
 }
 
+static int vma_get_mapfile_flags(struct vma_area *vma, DIR *mfd, char *path)
+{
+	struct stat stat;
+
+	if (fstatat(dirfd(mfd), path, &stat, AT_SYMLINK_NOFOLLOW) < 0) {
+		if (errno == ENOENT) {
+			/* Just mapping w/o map_files link */
+			return 0;
+		}
+		pr_perror("Failed fstatat on map %"PRIx64"", vma->e->start);
+		return -1;
+	}
+
+	switch(stat.st_mode & 0600) {
+		case 0200:
+			vma->e->fdflags = O_WRONLY;
+			break;
+		case 0400:
+			vma->e->fdflags = O_RDONLY;
+			break;
+		case 0600:
+			vma->e->fdflags = O_RDWR;
+			break;
+	}
+	vma->e->has_fdflags = true;
+	return 0;
+}
+
 static int vma_get_mapfile(char *fname, struct vma_area *vma, DIR *mfd,
-		struct vma_file_info *vfi, struct vma_file_info *prev_vfi)
+			   struct vma_file_info *vfi,
+			   struct vma_file_info *prev_vfi,
+			   int *vm_file_fd)
 {
 	char path[32];
 	int flags;
 
+	/* Figure out if it's file mapping */
+	snprintf(path, sizeof(path), "%"PRIx64"-%"PRIx64, vma->e->start, vma->e->end);
+
+	if (vma_get_mapfile_flags(vma, mfd, path))
+		return -1;
+
 	if (prev_vfi->vma && vfi_equal(vfi, prev_vfi)) {
 		struct vma_area *prev = prev_vfi->vma;
 
@@ -188,12 +228,11 @@ static int vma_get_mapfile(char *fname, struct vma_area *vma, DIR *mfd,
 		 * If vfi is equal (!) and negative @vm_file_fd --
 		 * we have nothing to borrow for sure.
 		 */
-		if (prev->vm_file_fd < 0)
+		if (*vm_file_fd < 0)
 			return 0;
 
 		pr_debug("vma %"PRIx64" borrows vfi from previous %"PRIx64"\n",
 				vma->e->start, prev->e->start);
-		vma->vm_file_fd = prev->vm_file_fd;
 		if (prev->e->status & VMA_AREA_SOCKET)
 			vma->e->status |= VMA_AREA_SOCKET | VMA_AREA_REGULAR;
 
@@ -208,9 +247,7 @@ static int vma_get_mapfile(char *fname, struct vma_area *vma, DIR *mfd,
 
 		return 0;
 	}
-
-	/* Figure out if it's file mapping */
-	snprintf(path, sizeof(path), "%"PRIx64"-%"PRIx64, vma->e->start, vma->e->end);
+	close_safe(vm_file_fd);
 
 	/*
 	 * Note that we "open" it in dumper process space
@@ -228,8 +265,8 @@ static int vma_get_mapfile(char *fname, struct vma_area *vma, DIR *mfd,
 		 */
 		flags = O_RDONLY;
 
-	vma->vm_file_fd = openat(dirfd(mfd), path, flags);
-	if (vma->vm_file_fd < 0) {
+	*vm_file_fd = openat(dirfd(mfd), path, flags);
+	if (*vm_file_fd < 0) {
 		if (errno == ENOENT)
 			/* Just mapping w/o map_files link */
 			return 0;
@@ -249,8 +286,7 @@ static int vma_get_mapfile(char *fname, struct vma_area *vma, DIR *mfd,
 
 			if ((buf.st_mode & S_IFMT) == 0 && !strncmp(fname, AIO_FNAME, sizeof(AIO_FNAME) - 1)) {
 				/* AIO ring, let's try */
-				close(vma->vm_file_fd);
-				vma->aio_nr_req = -1;
+				close_safe(vm_file_fd);
 				vma->e->status = VMA_AREA_AIORING;
 				return 0;
 			}
@@ -275,7 +311,7 @@ static int vma_get_mapfile(char *fname, struct vma_area *vma, DIR *mfd,
 				/*
 				 * Another bad thing is that kernel first checks
 				 * for permission access to ANY map_files link,
-				 * then checks for its existance. So we have to
+				 * then checks for its existence. So we have to
 				 * check for file path being empty to "emulate"
 				 * the ENOENT case.
 				 */
@@ -327,7 +363,7 @@ static int vma_get_mapfile(char *fname, struct vma_area *vma, DIR *mfd,
 			}
 
 			if (fstat(fd, vma->vmst) < 0) {
-				pr_perror("Can't stat [%s]\n", fname);
+				pr_perror("Can't stat [%s]", fname);
 				close(fd);
 				return -1;
 			}
@@ -340,7 +376,7 @@ static int vma_get_mapfile(char *fname, struct vma_area *vma, DIR *mfd,
 				return -1;
 			}
 
-			vma->vm_file_fd = fd;
+			*vm_file_fd = fd;
 			return 0;
 		}
 
@@ -364,14 +400,14 @@ static int vma_get_mapfile(char *fname, struct vma_area *vma, DIR *mfd,
 	if (opts.aufs) {
 		int ret;
 
-		ret = fixup_aufs_vma_fd(vma);
+		ret = fixup_aufs_vma_fd(vma, *vm_file_fd);
 		if (ret < 0)
 			return -1;
 		if (ret > 0)
 			return 0;
 	}
 
-	if (fstat(vma->vm_file_fd, vma->vmst) < 0) {
+	if (fstat(*vm_file_fd, vma->vmst) < 0) {
 		pr_perror("Failed fstat on map %"PRIx64"", vma->e->start);
 		return -1;
 	}
@@ -382,31 +418,44 @@ static int vma_get_mapfile(char *fname, struct vma_area *vma, DIR *mfd,
 int parse_self_maps_lite(struct vm_area_list *vms)
 {
 	FILE *maps;
+	struct vma_area *prev = NULL;
 
 	vm_area_list_init(vms);
 
 	maps = fopen_proc(PROC_SELF, "maps");
-	if (maps == NULL) {
-		pr_perror("Can't open self maps");
+	if (maps == NULL)
 		return -1;
-	}
 
 	while (fgets(buf, BUF_SIZE, maps) != NULL) {
 		struct vma_area *vma;
 		char *end;
+		unsigned long s, e;
 
-		vma = alloc_vma_area();
-		if (!vma) {
-			fclose(maps);
-			return -1;
+		s = strtoul(buf, &end, 16);
+		e = strtoul(end + 1, NULL, 16);
+
+		if (prev && prev->e->end == s)
+			/*
+			 * This list is needed for one thing only -- to
+			 * get the idea of what parts of current address
+			 * space are busy. So merge them altogether.
+			 */
+			prev->e->end = e;
+		else {
+			vma = alloc_vma_area();
+			if (!vma) {
+				fclose(maps);
+				return -1;
+			}
+
+			vma->e->start = s;
+			vma->e->end = e;
+			list_add_tail(&vma->list, &vms->h);
+			vms->nr++;
+			prev = vma;
 		}
 
-		vma->e->start = strtoul(buf, &end, 16);
-		vma->e->end = strtoul(end + 1, NULL, 16);
-		list_add_tail(&vma->list, &vms->h);
-		vms->nr++;
-
-		pr_debug("Parsed %"PRIx64"-%"PRIx64" vma\n", vma->e->start, vma->e->end);
+		pr_debug("Parsed %"PRIx64"-%"PRIx64" vma\n", prev->e->start, prev->e->end);
 	}
 
 	fclose(maps);
@@ -447,9 +496,11 @@ static int handle_vma(pid_t pid, struct vma_area *vma_area,
 			char *file_path, DIR *map_files_dir,
 			struct vma_file_info *vfi,
 			struct vma_file_info *prev_vfi,
-			struct vm_area_list *vma_area_list)
+			struct vm_area_list *vma_area_list,
+			int *vm_file_fd)
 {
-	if (vma_get_mapfile(file_path, vma_area, map_files_dir, vfi, prev_vfi))
+	if (vma_get_mapfile(file_path, vma_area, map_files_dir,
+					vfi, prev_vfi, vm_file_fd))
 		goto err_bogus_mapfile;
 
 	if (vma_area->e->status != 0) {
@@ -488,7 +539,7 @@ static int handle_vma(pid_t pid, struct vma_area *vma_area,
 		vma_area->e->shmid = prev->e->shmid;
 		vma_area->vmst = prev->vmst;
 		vma_area->mnt_id = prev->mnt_id;
-	} else if (vma_area->vm_file_fd >= 0) {
+	} else if (*vm_file_fd >= 0) {
 		struct stat *st_buf = vma_area->vmst;
 
 		if (S_ISREG(st_buf->st_mode))
@@ -530,7 +581,7 @@ static int handle_vma(pid_t pid, struct vma_area *vma_area,
 		 * have mnt_id.
 		 */
 		if (vma_area->mnt_id != -1 &&
-		    get_fd_mntid(vma_area->vm_file_fd, &vma_area->mnt_id))
+		    get_fd_mntid(*vm_file_fd, &vma_area->mnt_id))
 			return -1;
 	} else {
 		/*
@@ -551,7 +602,7 @@ err:
 err_bogus_mapping:
 	pr_err("Bogus mapping 0x%"PRIx64"-0x%"PRIx64" (flags: %#x vm_file_fd: %d)\n",
 	       vma_area->e->start, vma_area->e->end,
-	       vma_area->e->flags, vma_area->vm_file_fd);
+	       vma_area->e->flags, *vm_file_fd);
 	goto err;
 
 err_bogus_mapfile:
@@ -583,7 +634,13 @@ static int vma_list_add(struct vma_area *vma_area,
 
 		pages = vma_area_len(vma_area) / PAGE_SIZE;
 		vma_area_list->priv_size += pages;
-		vma_area_list->longest = max(vma_area_list->longest, pages);
+		vma_area_list->priv_longest = max(vma_area_list->priv_longest, pages);
+	} else if (vma_area_is(vma_area, VMA_ANON_SHARED)) {
+		unsigned long pages;
+
+		pages = vma_area_len(vma_area) / PAGE_SIZE;
+		vma_area_list->shared_longest =
+			max(vma_area_list->shared_longest, pages);
 	}
 
 	*prev_vfi = *vfi;
@@ -592,12 +649,13 @@ static int vma_list_add(struct vma_area *vma_area,
 	return 0;
 }
 
-int parse_smaps(pid_t pid, struct vm_area_list *vma_area_list)
+int parse_smaps(pid_t pid, struct vm_area_list *vma_area_list,
+					dump_filemap_t dump_filemap)
 {
 	struct vma_area *vma_area = NULL;
 	unsigned long start, end, pgoff, prev_end = 0;
 	char r, w, x, s;
-	int ret = -1;
+	int ret = -1, vm_file_fd = -1;
 	struct vma_file_info vfi;
 	struct vma_file_info prev_vfi = {};
 
@@ -606,8 +664,9 @@ int parse_smaps(pid_t pid, struct vm_area_list *vma_area_list)
 
 	vma_area_list->nr = 0;
 	vma_area_list->nr_aios = 0;
-	vma_area_list->longest = 0;
+	vma_area_list->priv_longest = 0;
 	vma_area_list->priv_size = 0;
+	vma_area_list->shared_longest = 0;
 	INIT_LIST_HEAD(&vma_area_list->h);
 
 	f.fd = open_proc(pid, "smaps");
@@ -692,8 +751,14 @@ int parse_smaps(pid_t pid, struct vm_area_list *vma_area_list)
 		}
 
 		if (handle_vma(pid, vma_area, str + path_off, map_files_dir,
-					&vfi, &prev_vfi, vma_area_list))
+				&vfi, &prev_vfi, vma_area_list, &vm_file_fd))
 			goto err;
+
+		if (vma_entry_is(vma_area->e, VMA_FILE_PRIVATE) ||
+				vma_entry_is(vma_area->e, VMA_FILE_SHARED)) {
+			if (dump_filemap && dump_filemap(vma_area, vm_file_fd))
+				goto err;
+		}
 	}
 
 	vma_area = NULL;
@@ -702,6 +767,7 @@ int parse_smaps(pid_t pid, struct vm_area_list *vma_area_list)
 err:
 	bclose(&f);
 err_n:
+	close_safe(&vm_file_fd);
 	if (map_files_dir)
 		closedir(map_files_dir);
 
@@ -920,10 +986,8 @@ int parse_pid_status(pid_t pid, struct proc_status_creds *cr)
 	bool parsed_seccomp = false;
 
 	f.fd = open_proc(pid, "status");
-	if (f.fd < 0) {
-		pr_perror("Can't open proc status");
+	if (f.fd < 0)
 		return -1;
-	}
 
 	cr->sigpnd = 0;
 	cr->shdpnd = 0;
@@ -1130,7 +1194,7 @@ static int parse_mnt_flags(char *opt, unsigned *flags)
 
 	/* Otherwise the kernel assumes RELATIME by default */
 	if ((*flags & (MS_RELATIME | MS_NOATIME)) == 0)
-		*flags = MS_STRICTATIME;
+		*flags |= MS_STRICTATIME;
 
 	return 0;
 }
@@ -1228,7 +1292,6 @@ static int parse_mountinfo_ent(char *str, struct mount_info *new, char **fsname)
 	new->mountpoint = xmalloc(PATH_MAX);
 	if (new->mountpoint == NULL)
 		goto err;
-	new->ns_mountpoint = new->mountpoint;
 
 	new->mountpoint[0] = '.';
 	ret = sscanf(str, "%i %i %u:%u %ms %s %ms %n",
@@ -1251,6 +1314,8 @@ static int parse_mountinfo_ent(char *str, struct mount_info *new, char **fsname)
 	new->mountpoint = xrealloc(new->mountpoint, strlen(new->mountpoint) + 1);
 	if (!new->mountpoint)
 		goto err;
+	new->ns_mountpoint = new->mountpoint;
+	new->is_ns_root = is_root(new->ns_mountpoint + 1);
 
 	new->s_dev = new->s_dev_rt = MKKDEV(kmaj, kmin);
 	new->flags = 0;
@@ -1277,6 +1342,10 @@ static int parse_mountinfo_ent(char *str, struct mount_info *new, char **fsname)
 
 	cure_path(new->source);
 
+	new->fsname = xstrdup(*fsname);
+	if (!new->fsname)
+		goto err;
+
 	/*
 	 * The kernel reports "subtypes" sometimes and the valid
 	 * type-vs-subtype delimiter is the dot symbol. We disregard
@@ -1339,15 +1408,12 @@ struct mount_info *parse_mountinfo(pid_t pid, struct ns_id *nsid, bool for_dump)
 {
 	struct mount_info *list = NULL;
 	FILE *f;
-	char str[1024];
 
 	f = fopen_proc(pid, "mountinfo");
-	if (!f) {
-		pr_perror("Can't open %d mountinfo", pid);
+	if (!f)
 		return NULL;
-	}
 
-	while (fgets(str, sizeof(str), f)) {
+	while (fgets(buf, BUF_SIZE, f)) {
 		struct mount_info *new;
 		int ret = -1;
 		char *fsname = NULL;
@@ -1358,9 +1424,9 @@ struct mount_info *parse_mountinfo(pid_t pid, struct ns_id *nsid, bool for_dump)
 
 		new->nsid = nsid;
 
-		ret = parse_mountinfo_ent(str, new, &fsname);
+		ret = parse_mountinfo_ent(buf, new, &fsname);
 		if (ret < 0) {
-			pr_err("Bad format in %d mountinfo: '%s'\n", pid, str);
+			pr_err("Bad format in %d mountinfo: '%s'\n", pid, buf);
 			goto end;
 		}
 
@@ -1383,11 +1449,22 @@ struct mount_info *parse_mountinfo(pid_t pid, struct ns_id *nsid, bool for_dump)
 
 		if (new->fstype->parse) {
 			ret = new->fstype->parse(new);
-			if (ret) {
+			if (ret < 0) {
 				pr_err("Failed to parse FS specific data on %s\n",
 						new->mountpoint);
+				mnt_entry_free(new);
+				new = NULL;
 				goto end;
 			}
+
+			if (ret > 0) {
+				pr_info("\tskipping fs mounted at %s\n", new->mountpoint + 1);
+				mnt_entry_free(new);
+				new = NULL;
+				ret = 0;
+				goto end;
+
+			}
 		}
 end:
 		if (fsname)
@@ -1491,6 +1568,9 @@ static int parse_timerfd(struct bfd *f, char *str, TimerfdEntry *tfy)
 	if (sscanf(str, "clockid: %d", &tfy->clockid) != 1)
 		goto parse_err;
 
+	if (verify_timerfd(tfy) < 0)
+		goto parse_err;
+
 	str = breadline(f);
 	if (IS_ERR_OR_NULL(str))
 		goto nodata;
@@ -1540,10 +1620,8 @@ static int parse_fdinfo_pid_s(int pid, int fd, int type,
 	int ret, exit_code = -1;;
 
 	f.fd = open_proc(pid, "fdinfo/%d", fd);
-	if (f.fd < 0) {
-		pr_perror("Can't open fdinfo/%d to parse", fd);
+	if (f.fd < 0)
 		return -1;
-	}
 
 	if (bfdopenr(&f))
 		return -1;
@@ -1926,6 +2004,11 @@ static int parse_file_lock_buf(char *buf, struct file_lock *fl,
 	return 0;
 }
 
+static bool pid_in_pstree(pid_t pid)
+{
+	return pstree_item_by_real(pid) != NULL;
+}
+
 int parse_file_locks(void)
 {
 	struct file_lock *fl;
@@ -1938,10 +2021,8 @@ int parse_file_locks(void)
 		return 0;
 
 	fl_locks = fopen_proc(PROC_GEN, "locks");
-	if (!fl_locks) {
-		pr_perror("Can't open file locks file!");
+	if (!fl_locks)
 		return -1;
-	}
 
 	while (fgets(buf, BUF_SIZE, fl_locks)) {
 		is_blocked = strstr(buf, "->") != NULL;
@@ -2026,10 +2107,8 @@ int parse_posix_timers(pid_t pid, struct proc_posix_timers_stat *args)
 	args->timer_n = 0;
 
 	f.fd = open_proc(pid, "timers");
-	if (f.fd < 0) {
-		pr_perror("Can't open posix timers file!");
+	if (f.fd < 0)
 		return -1;
-	}
 
 	if (bfdopenr(&f))
 		return -1;
@@ -2141,6 +2220,7 @@ int parse_threads(int pid, struct pid **_t, int *_n)
 			t[nr - 1].virt = -1;
 		}
 		t[nr - 1].real = atoi(de->d_name);
+		t[nr - 1].state = TASK_THREAD;
 		nr++;
 	}
 
@@ -2185,6 +2265,18 @@ int parse_cgroup_file(FILE *f, struct list_head *retl, unsigned int *n)
 		if (e)
 			*e = '\0';
 
+		/*
+		 * Controllers and their props might be
+		 * configured the way some of them are
+		 * not taken into the image for migration
+		 * sake or container specifics.
+		 */
+		if (cgp_should_skip_controller(name)) {
+			pr_debug("cg-prop: Skipping controller %s\n", name);
+			xfree(ncc);
+			continue;
+		}
+
 		ncc->name = xstrdup(name);
 		ncc->path = xstrdup(path);
 		ncc->cgns_prefix = 0;
@@ -2219,10 +2311,8 @@ int parse_task_cgroup(int pid, struct parasite_dump_cgroup_args *args, struct li
 	struct cg_ctl *intern, *ext;
 
 	f = fopen_proc(pid, "cgroup");
-	if (!f) {
-		pr_perror("couldn't open task cgroup file");
+	if (!f)
 		return -1;
-	}
 
 	ret = parse_cgroup_file(f, retl, n);
 	fclose(f);
@@ -2238,7 +2328,7 @@ int parse_task_cgroup(int pid, struct parasite_dump_cgroup_args *args, struct li
 
 	f = fmemopen(args->contents, strlen(args->contents), "r");
 	if (!f) {
-		pr_perror("couldn't fmemopen cgroup buffer:\n%s\n", args->contents);
+		pr_perror("couldn't fmemopen cgroup buffer %s", args->contents);
 		return -1;
 	}
 
@@ -2302,6 +2392,7 @@ void put_ctls(struct list_head *l)
 		xfree(c->path);
 		xfree(c);
 	}
+	INIT_LIST_HEAD(l);
 }
 
 /* Parse and create all the real controllers. This does not include things with
@@ -2407,55 +2498,6 @@ int aufs_parse(struct mount_info *new)
 	return ret;
 }
 
-bool proc_status_creds_dumpable(struct proc_status_creds *parent,
-				struct proc_status_creds *child)
-{
-	const size_t size = sizeof(struct proc_status_creds) -
-			offsetof(struct proc_status_creds, cap_inh);
-
-	/*
-	 * The comparision rules are the following
-	 *
-	 *  - CAPs can be different
-	 *  - seccomp filters should be passed via
-	 *    semantic comparision (FIXME) but for
-	 *    now we require them to be exactly
-	 *    identical
-	 *  - the rest of members must match
-	 */
-
-	if (memcmp(parent, child, size)) {
-		if (!pr_quelled(LOG_DEBUG)) {
-			pr_debug("Creds undumpable (parent:child)\n"
-				 "  uids:               %d:%d %d:%d %d:%d %d:%d\n"
-				 "  gids:               %d:%d %d:%d %d:%d %d:%d\n"
-				 "  state:              %d:%d"
-				 "  ppid:               %d:%d\n"
-				 "  sigpnd:             %llu:%llu\n"
-				 "  shdpnd:             %llu:%llu\n"
-				 "  seccomp_mode:       %d:%d\n"
-				 "  last_filter:        %u:%u\n",
-				 parent->uids[0], child->uids[0],
-				 parent->uids[1], child->uids[1],
-				 parent->uids[2], child->uids[2],
-				 parent->uids[3], child->uids[3],
-				 parent->gids[0], child->gids[0],
-				 parent->gids[1], child->gids[1],
-				 parent->gids[2], child->gids[2],
-				 parent->gids[3], child->gids[3],
-				 parent->state, child->state,
-				 parent->ppid, child->ppid,
-				 parent->sigpnd, child->sigpnd,
-				 parent->shdpnd, child->shdpnd,
-				 parent->seccomp_mode, child->seccomp_mode,
-				 parent->last_filter, child->last_filter);
-		}
-		return false;
-	}
-
-	return true;
-}
-
 int parse_children(pid_t pid, pid_t **_c, int *_n)
 {
 	pid_t *ch = NULL;
